#ifndef _KERNEL_H_
#define _KERNEL_H_
#include <stdio.h>
#define BLOCOS 2  //Ateno para o tamanho do bloco (no pode ser maior que os dados a serem processados).

//------------------------------------------------------
//
//------------------------------------------------------
__global__ void SomaKernel(float *C, float *A, float *B)
{
	/*
	 *	blockIdx.x    --> ndice do bloco.
	 *  blockDim.x    --> tamanho do bloco.
	 *  threadIdx.x   --> ndice da thread.
	 */
	const unsigned int indice = 
		(blockIdx.x * blockDim.x) + threadIdx.x;

	C[indice] = A[indice] + B[indice];
}

//------------------------------------------------------
//
//------------------------------------------------------
extern "C" void SomaVetoresGPU
(float *vetC, float *vetA, float *vetB, int tamanho)
{
	//Variveis
	unsigned int iMem_size = tamanho * sizeof(float);
	
	dim3 Grid(tamanho / BLOCOS , 1, 1), 
	     thread(BLOCOS, 1, 1);

	float	*GPUvetC = NULL, 
			*GPUvetA = NULL, 
			*GPUvetB = NULL;

	//Alocando memria na GPU.
	cudaMalloc((void**) &GPUvetC, iMem_size);
	cudaMalloc((void**) &GPUvetA, iMem_size);
	cudaMalloc((void**) &GPUvetB, iMem_size);

	//Copiando dados para GPU.
	cudaMemcpy(GPUvetA, vetA, iMem_size, cudaMemcpyHostToDevice);
	cudaMemcpy(GPUvetB, vetB, iMem_size, cudaMemcpyHostToDevice);

	//Executando kernel.
	SomaKernel<<<Grid, thread>>>(GPUvetC, GPUvetA, GPUvetB);

	//Copiando resultado da GPU para CPU.
	cudaMemcpy(vetC, GPUvetC, iMem_size, cudaMemcpyDeviceToHost);

	//Desalocando memria da GPU.
	cudaFree(GPUvetC);
	cudaFree(GPUvetA);
	cudaFree(GPUvetB);


}

//------------------------------------------------------
//
//------------------------------------------------------
extern "C" void SomaVetoresCPU(float *vetC, float *vetA, float *vetB, int tamanho)
{
	int i;
	printf("\n\nSomando em CPU....\n\n");
	for (i = 0; i < tamanho; i++)
	{
		vetC[i] = vetA[i] + vetB[i];
	}
}

//------------------------------------------------------
//
//------------------------------------------------------
extern "C" void FuncaoEmCPU(void)
{
	printf("\nFuncao do arquivo %s", __FILE__);
}

#endif