/*
 * Demo sem CUDA.
 */
#include <windows.h>
#include <cstdio>
#include <cstdlib>
#include <cmath>
#include <vector>
#include <GL/glew.h>
#include <gl/gl.h>
#include <GL/glut.h>
extern "C"{
#include"image.h"
}

//CUDA includes
#include <vector_types.h>
#include <cuda.h>
#include <cuda_runtime_api.h>
#include <cuda_gl_interop.h>

//CUDA's libs
#pragma comment (lib, "cuda.lib")
#pragma comment (lib, "cudart.lib")
#pragma comment (lib, "glew32.lib")
#pragma comment (lib, "glut32.lib")
#pragma comment (lib, "opengl32.lib")
#pragma comment (lib, "glu32.lib")

tpImage *texture; 

#define TIPO 0
extern "C" void runGPU(uchar4 *buffer, unsigned int w, unsigned int h, int type);
//------------------------------------------------------
//
//------------------------------------------------------
void OnRender(void)
{

	uchar4 *buffer;  //uchar4 pq a imagem  RGBA (32bits)

	glClearColor(0.0f,0.0f, 0.0f, 1.0f);  
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);          

	glColor3f(1.0f, 1.0f, 1.0f);

	if( cudaSuccess != cudaGLMapBufferObject((void**)&buffer, texture->bufferObject))			//passo 2
	{
		fprintf(stderr, "\nError in cudaGLRegisterBufferObject");
		getc(stdin);
		exit(EXIT_FAILURE);
	}

	runGPU(buffer, texture->width, texture->height, TIPO); 


	if( cudaSuccess != cudaGLUnmapBufferObject(texture->bufferObject))							//passo 3
	{
		fprintf(stderr, "\nError in cudaGLUnmapBufferObject r");
		getc(stdin);
		exit(EXIT_FAILURE);

	}
	
	glTexSubImage2D( GL_TEXTURE_2D, 0, 0, 0, texture->width,  texture->height, GL_RGBA, GL_UNSIGNED_BYTE, NULL);
	glBindTexture(GL_TEXTURE_2D, texture->textureHandle);
	glPushMatrix();

	

	  glBegin(GL_POLYGON);

		 glTexCoord2f(0.0f, 0.0f);
		 glVertex3f(0.1f, 0.1f, 0.0f);

		 glTexCoord2f(1.0f, 0.0f);
		 glVertex3f(0.9f, 0.1f, 0.0f);

		 glTexCoord2f(1.0f, 1.0f);
		 glVertex3f(0.9f, 0.9f, 0.0f);

		 glTexCoord2f(0.0f, 1.0f);
		 glVertex3f(0.1f, 0.9f, 0.0f);
	  glEnd();

	   glPopMatrix();
	  

	  
      
}

//------------------------------------------------------
//
//------------------------------------------------------
void OnShutDown(void)
{
	cudaGLUnregisterBufferObject(texture->bufferObject);								//passo 4
	DestroyImage(texture);
	exit(EXIT_SUCCESS);
}

//------------------------------------------------------
//
//------------------------------------------------------
void OnKeyboard(unsigned char key, int x, int y)
{
	switch (key)
	{
        case 27: OnShutDown();break;
		
	}
}   

//------------------------------------------------------
//
//------------------------------------------------------
void Gameloop(void)
{   
   OnRender(); 
   glutSwapBuffers();
   glFlush();
}

//------------------------------------------------------
//
//------------------------------------------------------
void OnResized(int w, int h)
{
   glViewport(0, 0, w, h);  
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();

   glOrtho (0.0f, 1.0f, 0.0f, 1.0f, -10.0f, 10.0f);
   gluLookAt(0.0f, 0.0f, 0.0f, 0.0, 0.0, -10.0, 0.0, 1.0, 0.0);
   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();
}

//------------------------------------------------------
//
//------------------------------------------------------
int main (int argc, char **argv)
{

	//init glut ---------------------------------------------------------------------------------------------------------------------------------
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGBA | GLUT_DEPTH);
	glutInitWindowSize(800, 600);
	glutInitWindowPosition(10, 10);
	glutCreateWindow("GPU & OpenGL by Marcelo Zamith - Media Lab - 2009");

	//init glew ---------------------------------------------------------------------------------------------------------------------------------
	printf("Loading extensions: %s\n", glewGetErrorString(glewInit()));
	if(!glewIsSupported(
	  "GL_VERSION_2_0 " 
	  "GL_ARB_pixel_buffer_object "
	  "GL_EXT_framebuffer_object "
	)){
	  fprintf(stderr, "ERROR: Support for necessary OpenGL extensions missing.");
	  fflush(stderr);
	  return EXIT_FAILURE;
	}
	printf("OpenGL window created.\n");
	glEnable(GL_TEXTURE_2D);
	glEnable(GL_DEPTH_TEST);

	//textura
	texture = ReadPpmImage2RGBA("IMG2.ppm");
	
	CreateTexture(texture,GL_LINEAR,GL_LINEAR); 
	if( cudaSuccess != cudaGLRegisterBufferObject(texture->bufferObject))							//passo 1
    {
      fprintf(stderr, "\nError in cudaGLRegisterBufferObject");
      getc(stdin);
	  exit(EXIT_FAILURE);
    }
	glBindTexture(GL_TEXTURE_2D, texture->textureHandle);

	glutDisplayFunc(OnRender);
	glutReshapeFunc(OnResized);
	glutKeyboardFunc(OnKeyboard); 
	glutIdleFunc(Gameloop);
	glutMainLoop();
	OnShutDown();
	return EXIT_SUCCESS;
}